#pragma once
#ifndef INPUT_PARSER_H 
#define INPUT_PARSER_H

#include <istream>
#include <vector>
#include <set>
#include <string>

namespace lxs {
namespace input {

    /**
     * Used for parsing token rules
     */
    class InputParser {
        public:
            /**
             * parse the tokens rules read from `is`
             *
             * std::vector<std::pair<int, std::pair<std::string,std::string>>>
             *                     <line number, <tokenname, regex> >
             */
            static std::vector<std::pair<int, std::pair<std::string,std::string>>> parseInput(std::istream& is);
            
            /**
             * Get a list of the tokens specified in the istream 
             *
             * @param is The istream to get the tokens from
             *
             * @return the list of tokens
             */
            static std::set<std::string> getTokens(std::istream& is);
    };
    
    /**
     * Used to throw errors when the inputfile was not valid
     */

    class InputParserException: public std::exception {
    public:
        InputParserException(std::string what);
        virtual const char* what() const throw();

    private:
        std::string m_what;
    };
}
}
#endif // INPUT_PARSER_H
